@@ -1,5 +1,5 @@
 #  Phusion Passenger - https://www.phusionpassenger.com/
-#  Copyright (c) 2010-2014 Phusion Holding B.V.
+#  Copyright (c) 2010-2018 Phusion Holding B.V.
 #
 #  "Passenger", "Phusion Passenger" and "Union Station" are registered
 #  trademarks of Phusion Holding B.V.
@@ -141,6 +141,15 @@ module PhusionPassenger
     end
     private_class_method :cc_or_cxx_supports_feliminate_unused_debug?
 
+    def self.cc_or_cxx_supports_blocks?(language)
+      ext = detect_language_extension(language)
+      compiler_type_name = detect_compiler_type_name(language)
+      command = create_compiler_command(language,"-E -dM",'- </dev/null')
+      result = `#{command}`
+      return result.include? "__BLOCKS__"
+    end
+    private_class_method :cc_or_cxx_supports_blocks?
+
   public
     def self.cc
       return string_env('CC', default_cc)
@@ -189,12 +198,12 @@ module PhusionPassenger
     memoize :cxx_is_gcc?
 
     def self.cc_is_clang?
-      `#{cc} --version 2>&1` =~ /clang version/
+      `#{cc} --version 2>&1` =~ /clang( version|-)/
     end
     memoize :cc_is_clang?
 
     def self.cxx_is_clang?
-      `#{cxx} --version 2>&1` =~ /clang version/
+      `#{cxx} --version 2>&1` =~ /clang( version|-)/
     end
     memoize :cxx_is_clang?
 
@@ -270,7 +279,7 @@ module PhusionPassenger
           "-c '#{filename}' -o '#{filename}.o'",
           flags)
         result = run_compiler(description, command, filename, source, true)
-        result = result && result[:result] && result[:output] !~ /unknown warning option/i
+        result = result && result[:result] && result[:output] !~ /unknown warning option|unrecognized command line option/i
       end
       return false if !result
 
@@ -344,6 +353,13 @@ module PhusionPassenger
     end
     memoize :cc_supports_visibility_flag?, true
 
+    def self.cc_supports_fno_limit_debug_info_flag?
+      try_compile_with_warning_flag(
+        "Checking for C compiler '-fno-limit-debug-info' support",
+        :c, '', '-fno-limit-debug-info')
+    end
+    memoize :cc_supports_fno_limit_debug_info_flag?
+
     def self.cxx_supports_visibility_flag?
       return false if os_name_simple == "aix"
       return try_compile("Checking for C++ compiler '-fvisibility' support",
@@ -386,6 +402,20 @@ module PhusionPassenger
     end
     memoize :cxx_supports_wno_unused_local_typedefs_flag?, true
 
+    def self.cxx_supports_wno_format_nonliteral_flag?
+      return try_compile_with_warning_flag(
+        "Checking for C++ compiler '-Wno-format-nonliteral' support",
+        :cxx, '', '-Wno-format-nonliteral')
+    end
+    memoize :cxx_supports_wno_format_nonliteral_flag?, true
+
+    def self.cxx_supports_fno_limit_debug_info_flag?
+      try_compile_with_warning_flag(
+        "Checking for C++ compiler '-fno-limit-debug-info' support",
+        :cxx, '', '-fno-limit-debug-info')
+    end
+    memoize :cxx_supports_fno_limit_debug_info_flag?
+
     def self.cc_supports_no_tls_direct_seg_refs_option?
       return try_compile("Checking for C compiler '-mno-tls-direct-seg-refs' support",
         :c, '', '-mno-tls-direct-seg-refs')
@@ -415,6 +445,16 @@ module PhusionPassenger
     end
     memoize :cxx_supports_feliminate_unused_debug?, true
 
+    def self.cc_block_support_ok?
+      return (os_name_simple != 'macosx' || cc_or_cxx_supports_blocks?(:c) || os_version >= "10.13" )
+    end
+    memoize :cc_block_support_ok?, true
+
+    def self.cxx_block_support_ok?
+      return (os_name_simple != 'macosx' || cc_or_cxx_supports_blocks?(:cxx) || os_version >= "10.13" )
+    end
+    memoize :cxx_block_support_ok?, true
+
     # Returns whether compiling C++ with -fvisibility=hidden might result
     # in tons of useless warnings, like this:
     # http://code.google.com/p/phusion-passenger/issues/detail?id=526
@@ -439,18 +479,18 @@ module PhusionPassenger
     end
     memoize :cxx_visibility_flag_generates_warnings?, true
 
-    def self.adress_sanitizer_flag
+    def self.address_sanitizer_flag
       if cc_is_clang?
         if `#{cc} --help` =~ /-fsanitize=/
-          return "-fsanitize=address"
+          "-fsanitize=address"
         else
-          return "-faddress-sanitizer"
+          "-faddress-sanitizer"
         end
       else
-        return nil
+        nil
       end
     end
-    memoize :adress_sanitizer_flag
+    memoize :address_sanitizer_flag
 
     def self.cxx_11_flag
       # C++11 support on FreeBSD 10.0 + Clang seems to be bugged.
@@ -484,6 +524,12 @@ module PhusionPassenger
     end
     memoize :has_math_library?, true
 
+    def self.has_dl_library?
+      return try_link("Checking for -ldl support",
+        :c, "int main() { return 0; }\n", '-ldl')
+    end
+    memoize :has_dl_library?, true
+
     def self.has_alloca_h?
       return try_compile("Checking for alloca.h",
         :c, '#include <alloca.h>')
@@ -507,44 +553,33 @@ module PhusionPassenger
       #
       # In any case we'll always want to use -ggdb for better GDB debugging.
       if cc_is_gcc?
-        return '-ggdb'
+        result = '-ggdb'
       else
-        return '-g'
+        result = '-g'
       end
+      if cc_supports_fno_limit_debug_info_flag?
+        result << ' -fno-limit-debug-info'
+      end
+      result
     end
 
-    def self.dmalloc_ldflags
-      if !ENV['DMALLOC_LIBS'].to_s.empty?
-        return ENV['DMALLOC_LIBS']
-      end
-      if os_name_simple == "macosx"
-        ['/opt/local', '/usr/local', '/usr'].each do |prefix|
-          filename = "#{prefix}/lib/libdmallocthcxx.a"
-          if File.exist?(filename)
-            return filename
-          end
-        end
-        return nil
+    # C++ compiler flags that should be passed in order to enable debugging information.
+    def self.debugging_cxxflags
+      # According to OpenBSD's pthreads man page, pthreads do not work
+      # correctly when an app is compiled with -g. It recommends using
+      # -ggdb instead.
+      #
+      # In any case we'll always want to use -ggdb for better GDB debugging.
+      if cc_is_gcc?
+        result = '-ggdb'
       else
-        return "-ldmallocthcxx"
+        result = '-g'
       end
-    end
-    memoize :dmalloc_ldflags
-
-    def self.electric_fence_ldflags
-      if os_name_simple == "macosx"
-        ['/opt/local', '/usr/local', '/usr'].each do |prefix|
-          filename = "#{prefix}/lib/libefence.a"
-          if File.exist?(filename)
-            return filename
-          end
-        end
-        return nil
-      else
-        return "-lefence"
+      if cxx_supports_fno_limit_debug_info_flag?
+        result << ' -fno-limit-debug-info'
       end
+      result
     end
-    memoize :electric_fence_ldflags
 
     def self.export_dynamic_flags
       if os_name_simple == "linux"
